@@ -278,6 +278,34 @@ def parallel(*fns):
   _.__name__ = "parallel({}) ".format((',').join([f.__name__ for f in fns]))
   return _
 
+# Original source
+# http://code.activestate.com/recipes/577452-a-memoize-decorator-for-instance-methods/
+class memoized(object):
+  """Decorator that caches a function's return value each time it is called.
+  If called later with the same arguments, the cached value is returned, and
+  not re-evaluated.
+  """
+  def __init__(self, func):
+    self.func = func
+    self.cache = {}
+  def __call__(self, *args):
+    try:
+      return self.cache[args]
+    except KeyError:
+      value = self.func(*args)
+      self.cache[args] = value
+      return value
+    except TypeError:
+      # uncachable -- for instance, passing a list as an argument.
+      # Better to not cache than to blow up entirely.
+      return self.func(*args)
+  def __repr__(self):
+    """Return the function's docstring."""
+    return self.func.__doc__
+  def __get__(self, obj, objtype):
+    """Support instance methods."""
+    return functools.partial(self.__call__, obj)
+
 ## parsing routines #######
 
 COMPARISON_OPS = {
